/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.cluster.jk;

import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.util.Properties;

import org.ow2.jasmine.rules.cluster.api.ServerState;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Provide some action to reconfigure JK.
 * @author Loris Bouzonnet
 */
public class JKAction {

    /**
     * Singleton.
     */
    private static JKAction jkAction;

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(JKAction.class);

    /**
     * manager webapp's encoding.
     */
    private static String CHARSET = "utf-8";

    private String webServerHost = "localhost";

    private int webServerPort = 80;

    private String statusPath = "/jkmanager";

    private String user = "me";

    private String pass = "xx";


    /**
     * No public constructor.
     */
    private JKAction() {
        // Do nothing
    }

    /**
     * @return the only instance of JKAction.
     */
    public static JKAction getJKAction() {
        if(jkAction == null) {
            jkAction = new JKAction();
        }
        return jkAction;
    }

    /**
     * Return state of the worker or null if it cannot be retrieved.
     * @return state of the worker or null if it cannot be retrieved
     * @throws Exception
     */
    public ServerState getServerState(final String serverName) throws Exception {
        Properties props = getJKProperties();
        boolean blacklisted = props.getProperty("worker." + serverName + ".activation").equals("DIS");
        int loadfactor = Integer.parseInt(props.getProperty("worker." + serverName + ".lbfactor"));
        return new ServerState(serverName, blacklisted, loadfactor);
    }

    public String getWebServerHost() {
        return webServerHost;
    }

    public void setWebServerHost(String webServerHost) {
        this.webServerHost = webServerHost;
    }

    public int getWebServerPort() {
        return webServerPort;
    }

    public void setWebServerPort(int webServerPort) {
        this.webServerPort = webServerPort;
    }

    public String getUser() {
        return user;
    }

    public void setUser(String user) {
        this.user = user;
    }

    public String getPass() {
        return pass;
    }

    public void setPass(String pass) {
        this.pass = pass;
    }

    public String getStatusPath() {
        return statusPath;
    }

    public void setStatusPath(String statusPath) {
        this.statusPath = statusPath;
    }


    /**
     * @return URL to Apache webserver.
     */
    private URL getJKStatusURL(String query) throws Exception {
        String path = statusPath + "?" + query;
        URL url = new URL("http", webServerHost, webServerPort, path);
        return url;
    }

    public void setLoadFactor(String serverName, int loadfactor) throws Exception {
        // Example: http://localhost/jkmanager?cmd=update&w=loadbalancer&sw=worker1&wf=9
        execute(getJKStatusURL("cmd=update&w=loadbalancer&sw=" + serverName + "&vwf=" + loadfactor), null);
    }

    public void blacklistMe(String serverName) throws Exception {
        // Example: http://localhost/jkmanager?cmd=update&w=loadbalancer&sw=worker1&wa=d
        execute(getJKStatusURL("cmd=update&w=loadbalancer&sw=" + serverName + "&vwa=d"), null);
    }

    public void unblacklistMe(String serverName) throws Exception {
        // Example: http://localhost/jkmanager?cmd=update&w=loadbalancer&sw=worker1&wa=d
        execute(getJKStatusURL("cmd=update&w=loadbalancer&sw=" + serverName + "&vwa=a"), null);
    }

    private Properties getJKProperties() throws Exception {
        Properties props = new Properties();
        execute(getJKStatusURL("mime=prop"), props);
        return props;
    }

    private void execute(URL url, Object result) throws Exception {
        URLConnection conn = null;
        InputStreamReader reader = null;
        try {

            // Create a connection for this command
            conn = url.openConnection();
            HttpURLConnection hconn = (HttpURLConnection) conn;

            // Set up standard connection characteristics
            hconn.setAllowUserInteraction(false);
            hconn.setDoInput(true);
            hconn.setUseCaches(false);
            hconn.setDoOutput(false);
            hconn.setRequestMethod("GET");
            hconn.setRequestProperty("User-Agent",
            "JASMINe-JK-Configurator/1.0");

            // Set up an authorization header with our credentials
            String input = user + ":" + pass;
            String output = new String(Base64.encode(input.getBytes()));
            hconn.setRequestProperty("Authorization",
                    "Basic " + output);

            // Establish the connection with the server
            hconn.connect();

            // Process the response message
            reader = new InputStreamReader(hconn.getInputStream(), CHARSET);
            StringBuffer buff = new StringBuffer();
            while (true) {
                int ch = reader.read();
                if (ch < 0) {
                    break;
                } else if ((ch == '\r') || (ch == '\n')) {
                    // in Win \r\n would cause handleOutput() to be called
                    // twice, the second time with an empty string,
                    // producing blank lines
                    if (buff.length() > 0) {
                        String line = buff.toString();
                        buff.setLength(0);
                        handleOutput(line, result);
                    }
                } else {
                    buff.append((char) ch);
                }
            }
            if (buff.length() > 0) {
                handleOutput(buff.toString(), result);
            }
        } catch (Throwable t) {
            throw new Exception(t);
        } finally {
            if (reader != null) {
                try {
                    reader.close();
                } catch (Throwable u) {
                    ;
                }
                reader = null;
            }
        }
    }

    private void handleOutput(String line, Object result) {
        logger.debug("{0}", line);
        if(result != null && result instanceof Properties){
            String[] keyValue = line.split("=");
            if(keyValue.length == 2) {
                ((Properties) result).setProperty(keyValue[0], keyValue[1]);
            }
        }
    }

    /**
     * For testing purpose.
     */
    public static void main(String[] args) {
        JKAction jkAction = JKAction.getJKAction();
        jkAction.setStatusPath("/jkmanager");
        jkAction.setWebServerHost("localhost");
        jkAction.setWebServerPort(80);
        try {
            Properties props = jkAction.getJKProperties();
            System.out.println("worker.node1.lbfactor=" + props.get("worker.node1.lbfactor"));
            jkAction.setLoadFactor("node1", 07);
            props = jkAction.getJKProperties();
            System.out.println("worker.node1.lbfactor=" + props.get("worker.node1.lbfactor"));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
